'use strict';

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _osFamily = require('os-family');

var _osFamily2 = _interopRequireDefault(_osFamily);

var _os = require('os');

var _utilsDelay = require('../utils/delay');

var _utilsDelay2 = _interopRequireDefault(_utilsDelay);

var _utilsExec = require('../utils/exec');

var _binaries = require('../binaries');

var _binaries2 = _interopRequireDefault(_binaries);

/**
 * Retrieves a platform-specific window descriptor for the window that contains a web page with the specified title.
 * @function
 * @async
 * @name findWindow
 * @param {string} pageTitle - The title of the web page opened in a window whose descriptor should be retrieved.
 * @returns {object} a platform-specific window descriptor that can be used as a window identifier.
 **/
var GRANT_PERMISSIONS_RETRY_COUNT = 10;
var GRANT_PERMISSIONS_DELAY = 3000;
var GRANT_PERMISSIONS_EXIT_CODE = 2;

function runFindWindowBinary(pageTitle) {
    var i, code;
    return _regeneratorRuntime.async(function runFindWindowBinary$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                i = 0;

            case 1:
                if (!(i < GRANT_PERMISSIONS_RETRY_COUNT)) {
                    context$1$0.next = 19;
                    break;
                }

                context$1$0.prev = 2;
                context$1$0.next = 5;
                return _regeneratorRuntime.awrap(_utilsExec.execFile(_binaries2.default.findWindow, [pageTitle]));

            case 5:
                return context$1$0.abrupt('return', context$1$0.sent);

            case 8:
                context$1$0.prev = 8;
                context$1$0.t0 = context$1$0['catch'](2);
                code = context$1$0.t0.status || context$1$0.t0.code;

                if (!(code === GRANT_PERMISSIONS_EXIT_CODE)) {
                    context$1$0.next = 15;
                    break;
                }

                context$1$0.next = 14;
                return _regeneratorRuntime.awrap(_utilsDelay2.default(GRANT_PERMISSIONS_DELAY));

            case 14:
                return context$1$0.abrupt('continue', 16);

            case 15:
                throw context$1$0.t0;

            case 16:
                i++;
                context$1$0.next = 1;
                break;

            case 19:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[2, 8]]);
}

exports.default = function callee$0$0(pageTitle) {
    var res, windowParams;
    return _regeneratorRuntime.async(function callee$0$0$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                res = null;
                windowParams = [];
                context$1$0.prev = 2;
                context$1$0.next = 5;
                return _regeneratorRuntime.awrap(runFindWindowBinary(pageTitle));

            case 5:
                res = context$1$0.sent;
                context$1$0.next = 11;
                break;

            case 8:
                context$1$0.prev = 8;
                context$1$0.t0 = context$1$0['catch'](2);
                return context$1$0.abrupt('return', null);

            case 11:

                windowParams = res.split(_os.EOL);

                if (!_osFamily2.default.win) {
                    context$1$0.next = 14;
                    break;
                }

                return context$1$0.abrupt('return', { hwnd: windowParams[0], browser: windowParams[1] });

            case 14:
                if (!_osFamily2.default.mac) {
                    context$1$0.next = 16;
                    break;
                }

                return context$1$0.abrupt('return', { processId: windowParams[0], cocoaId: windowParams[1], windowId: windowParams[2] });

            case 16:
                if (!_osFamily2.default.linux) {
                    context$1$0.next = 18;
                    break;
                }

                return context$1$0.abrupt('return', { windowId: windowParams[0] });

            case 18:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this, [[2, 8]]);
};

module.exports = exports.default;